using System;
using System.ComponentModel;
using Prodige.Drawing;
using Prodige.Drawing.Styles;

namespace Anchoring
{
	// A Picture demonstrating Anchoring, Docking, and relative scaling techniques for
	// positioning child Elements.

	public class LeftRightSample : Picture
	{
		private FSize _minimum;
		private FRectangle _bounds;	// needed to make Bounds independent of child Bounds
		private LeftRightLayout _leftRightLayout;
		private FloatLayout _floatLayout;

		public LeftRightSample()
		{
			InitializeComponent();
			_minimum = new FSize(_left.Width + _right.Width, _left.Height);
			// ignore all child Elements in initial _bounds computation,
			// except _left and _right
			_bounds = FRectangle.Union(_left.Bounds, _right.Bounds);
			_leftRightLayout = LeftRightLayout.Anchor;
			_floatLayout = FloatLayout.AnchorLeftTop;
		}

		[Category("Layout")]
		[Description("Layout of left and right rectangles.")]
		[DefaultValue(LeftRightLayout.Anchor)]
		public LeftRightLayout LeftRightLayout
		{
			get { return _leftRightLayout; }
			set { _leftRightLayout = value; }
		}

		[Category("Layout")]
		[Description("Layout of all children except left and right rectangles.")]
		[DefaultValue(FloatLayout.AnchorLeftTop)]
		public FloatLayout FloatLayout
		{
			get { return _floatLayout; }
			set { _floatLayout = value; }
		}

		protected override bool SetBoundsCore(FRectangle newBounds, BoundsChanges changes)
		{
			// clamp to minimum/maximum dimensions
			if (newBounds.Width < _minimum.Width)
				newBounds.Width = _minimum.Width;
			if (newBounds.Height < _minimum.Height)
				newBounds.Height = _minimum.Height;
			// check to see if no changes
			if (_bounds == newBounds)
				return false;

			// Perform resizing algorithm: all Elements must be taken care of.
			// Here we assume no Elements are transformed. If they were, we would need
			// to get/set a relative bounds instead (SetRelativeBounds).

			// handle anchored/docked Elements
			// whether anchored or docked, we position by upper corner
			_left.Location = newBounds.Location;
			_right.RightTop = newBounds.RightTop;
			if (_leftRightLayout == LeftRightLayout.Dock)
			{
				_left.Height = newBounds.Height;
				_right.Height = newBounds.Height;
			}

			// handle remaining Elements
			if (_floatLayout == FloatLayout.AnchorLeftTop)
			{
				// each child (except _left and _right) is moved by difference
				// between new and old upper left corner
				Vector delta = newBounds.Location - _bounds.Location;
				foreach (Element child in Elements)
				{
					if (child == _left || child == _right) continue;
					child.Location += delta;
				}
			}
			else
			{
				// Create a ScalingTranslation that transforms a rectangle from
				// the old _bounds to newBounds
				ScalingTranslation st = new ScalingTranslation(_bounds, newBounds);
				foreach (Element child in Elements)
				{
					if (child == _left || child == _right) continue;
					// For ScaleLeftTop, we scale and translate the upper left corner (Location).
					// For Scale, we scale and translate the whole child Bounds.
					if (_floatLayout == FloatLayout.ScaleLeftTop)
						child.Location = st.TransformPoint(child.Location);
					else if (_floatLayout == FloatLayout.Scale)
						child.Bounds *= st;
				}
			}
			_bounds = newBounds;
			return true;
		}

		protected override FRectangle GetBoundsCore()
		{
			// We override GetBoundsCore to ensure the Bounds property get value will
			// be independent of the child Bounds. By default, GetBoundsCore will return
			// the union of the child transformed Bounds.
			return _bounds;
		}

		#region Designer generated code
		/// <summary> 
		/// Required for designer support - do not modify with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this._left = new Prodige.Drawing.Rectangle();
			this._right = new Prodige.Drawing.Rectangle();
			this._floater = new Prodige.Drawing.Ellipse();
			((System.ComponentModel.ISupportInitialize)(this)).BeginInit();
			// 
			// _left
			// 
			this._left.CornerRadii = new Prodige.Drawing.Radii(16F, 16F);
			this._left.Location = new Prodige.Drawing.Vector(64F, 64F);
			this._left.Size = new Prodige.Drawing.FSize(64F, 64F);
			this._left.Text = "Left";
			// 
			// _right
			// 
			this._right.CornerRadii = new Prodige.Drawing.Radii(16F, 16F);
			this._right.Location = new Prodige.Drawing.Vector(192F, 64F);
			this._right.Size = new Prodige.Drawing.FSize(64F, 64F);
			this._right.Text = "Right";
			// 
			// _floater
			// 
			this._floater.Location = new Prodige.Drawing.Vector(128F, 64F);
			this._floater.Size = new Prodige.Drawing.FSize(64F, 64F);
			this._floater.Text = "Floats";
			// 
			// LeftRightSample
			// 
			this.Elements.AddRange(new Prodige.Drawing.Element[] {
																	 this._left,
																	 this._right,
																	 this._floater});
			((System.ComponentModel.ISupportInitialize)(this)).EndInit();

		}
	
		private Rectangle _left;
		private Ellipse _floater;
		private Rectangle _right;
		#endregion
	}
}
